"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var localization_error_1 = require("./localization_error");
/**
 * A service which provides localization for the web application.
 */
var LocalizationService = (function () {
    /**
     * Creates an instance of the service.
     * @param configuration The configuration of the localization.
     */
    function LocalizationService(configuration) {
        this._configuration = configuration;
    }
    /**
     * Translates the complete string.
     * @param stringForTranslation The string which should be translated.
     * @param languages The languages for which the translation should be found.
     * @param localizationObjects The objects that should contain the translations.
     */
    LocalizationService.prototype.TranslateWholeString = function (stringForTranslation, languages, localizationObjects) {
        var result = stringForTranslation;
        var regex = new RegExp(this._configuration.RegexString, "g");
        var keys;
        while ((keys = regex.exec(result)) !== null) {
            var translation = keys[1];
            try {
                translation = this.Translate(keys[1], languages, localizationObjects);
            }
            catch (error) {
            }
            finally {
                result = result.replace(keys[0], translation);
            }
        }
        return result;
    };
    /**
     * Translates a key.
     * @param key The key which should be translated.
     * @param languages The languages for which the translation should be found.
     * @param localizationObjects The objects that should contain the translation.
     */
    LocalizationService.prototype.Translate = function (key, languages, localizationObjects) {
        var orderedLanguages = languages.sort(function (a, b) {
            return b.Priority - a.Priority;
        });
        var translations = null;
        for (var _i = 0, localizationObjects_1 = localizationObjects; _i < localizationObjects_1.length; _i++) {
            var obj = localizationObjects_1[_i];
            if (obj.Key == key) {
                translations = obj.Translations;
                for (var _a = 0, orderedLanguages_1 = orderedLanguages; _a < orderedLanguages_1.length; _a++) {
                    var lang = orderedLanguages_1[_a];
                    for (var _b = 0, _c = obj.Translations; _b < _c.length; _b++) {
                        var translation = _c[_b];
                        if (translation.LanguageKey == lang.Key)
                            return translation.Translation;
                    }
                }
            }
        }
        try {
            if (translations == null)
                throw new localization_error_1.LocalizationError("Could not translate key!");
            return this.GetFallbackTranslation(translations);
        }
        catch (error) {
            if (error instanceof localization_error_1.LocalizationError)
                throw new localization_error_1.LocalizationError("Could not translate key!");
            throw error;
        }
    };
    /**
     * Gets the fallback translation.
     * @param translations The array of translation where the fallback translation should be found.
     */
    LocalizationService.prototype.GetFallbackTranslation = function (translations) {
        for (var _i = 0, translations_1 = translations; _i < translations_1.length; _i++) {
            var translation = translations_1[_i];
            if (translation.LanguageKey == this._configuration.FallbackLanguage)
                return translation.Translation;
        }
        throw new localization_error_1.LocalizationError("Fallback translation not found!");
    };
    return LocalizationService;
}());
exports.LocalizationService = LocalizationService;
